#!/bin/ksh

#**************************************************************
# Author: Sonia Labetoulle
# Contact: Sonia.Labetoulle__at__locean-ipsl.upmc.fr
# $Revision::                                          $ Revision of last commit
# $Author::                                            $ Author of last commit
# $Date::                                              $ Date of last commit
# IPSL (2012)
#  This software is governed by the CeCILL licence see libIGCM/libIGCM_CeCILL.LIC
#
#**************************************************************

function IGCM_ensemble_CastInit
{
  IGCM_debug_PushStack "IGCM_ensemble_CastInit"

  set -A Alpha      A B C D E F G H I J K L M N O P Q R S T U V W X Y Z
  set -A AlphaMonth a b c d e f g h i j k l

  IGCM_sys_Mkdir ${RUN_DIR}

  IGCM_sys_Cp ${SUBMIT_DIR}/config.card   ${RUN_DIR}
  IGCM_sys_Cp ${SUBMIT_DIR}/ensemble.card ${RUN_DIR}
  IGCM_sys_Cp ${SUBMIT_DIR}/Job_*         ${RUN_DIR}
  IGCM_sys_Cp ${SUBMIT_DIR}/run.card.init ${RUN_DIR}

  # Useful?
  #if [ -f  ${SUBMIT_DIR}/CreatedDir.txt ] ; then
  #  IGCM_sys_Cp ${SUBMIT_DIR}/CreatedDir.txt ${RUN_DIR}
  #fi
  # Useful?
  #if [ -f  ${SUBMIT_DIR}/Qsub.txt ] ; then
  #  IGCM_sys_Cp ${SUBMIT_DIR}/Qsub.txt ${RUN_DIR}
  #fi

  IGCM_card_DefineVariableFromOption ${FileName} Ens_PERTURB actif
  IGCM_card_DefineVariableFromOption ${FileName} Ens_PERTURB NAME
  IGCM_card_DefineVariableFromOption ${FileName} Ens_PERTURB DEBUT_INIT
  IGCM_card_DefineVariableFromOption ${FileName} Ens_PERTURB FIN_INIT
  IGCM_card_DefineVariableFromOption ${FileName} Ens_PERTURB PERIODICITE
  IGCM_card_DefineArrayFromOption    ${FileName} Ens_PERTURB NONPERIODIC
  IGCM_card_DefineVariableFromOption ${FileName} Ens_PERTURB DUREE
  IGCM_card_DefineArrayFromOption    ${FileName} Ens_PERTURB DUREE_NONPERIODIC
  IGCM_card_DefineVariableFromOption ${FileName} Ens_PERTURB MEMBER
  IGCM_card_DefineArrayFromOption    ${FileName} Ens_PERTURB PERTURB_BIN
  IGCM_card_DefineVariableFromOption ${FileName} Ens_PERTURB INITFROM
  IGCM_card_DefineVariableFromOption ${FileName} Ens_PERTURB INITPATH
  IGCM_card_DefineVariableFromOption config.card UserChoices TagName
  IGCM_card_DefineVariableFromOption config.card UserChoices CalendarType
  IGCM_card_DefineArrayFromSection   config.card ListOfComponents

  echo
  IGCM_debug_Print 1 "ACTIVE            = ${ensemble_Ens_PERTURB_actif}"
  IGCM_debug_Print 1 "NAME              = ${ensemble_Ens_PERTURB_NAME}"
  IGCM_debug_Print 1 "DEBUT_INIT        = ${ensemble_Ens_PERTURB_DEBUT_INIT}"
  IGCM_debug_Print 1 "FIN_INIT          = ${ensemble_Ens_PERTURB_FIN_INIT}"
  IGCM_debug_Print 1 "PERIODICITE       = ${ensemble_Ens_PERTURB_PERIODICITE}"
  IGCM_debug_Print 1 "NONPERIODIC       = ${ensemble_Ens_PERTURB_NONPERIODIC[*]}"
  IGCM_debug_Print 1 "DUREE             = ${ensemble_Ens_PERTURB_DUREE}"
  IGCM_debug_Print 1 "DUREE_NONPERIODIC = ${ensemble_Ens_PERTURB_DUREE_NONPERIODIC[*]}"
  IGCM_debug_Print 1 "MEMBER            = ${ensemble_Ens_PERTURB_MEMBER}"
  IGCM_debug_Print 1 "PERTURB_BIN       = ${ensemble_Ens_PERTURB_PERTURB_BIN[*]}"
  IGCM_debug_Print 1 "INITFROM          = ${ensemble_Ens_PERTURB_INITFROM}"
  IGCM_debug_Print 1 "INITPATH          = ${ensemble_Ens_PERTURB_INITPATH}"
  IGCM_debug_Print 1 "TagName           = ${config_UserChoices_TagName}"
  IGCM_debug_Print 1 "CalendarType      = ${config_UserChoices_CalendarType}"
  IGCM_debug_Print 1 "ListOfComponents  = ${config_ListOfComponents[*]}"

  PerturbExe=${ensemble_Ens_PERTURB_PERTURB_BIN[0]}
  PerturbComp=${ensemble_Ens_PERTURB_PERTURB_BIN[1]}
  PerturbFile=${ensemble_Ens_PERTURB_PERTURB_BIN[2]}
  PerturbVar=${ensemble_Ens_PERTURB_PERTURB_BIN[3]}
  PerturbAmp=${ensemble_Ens_PERTURB_PERTURB_BIN[4]}

  IGCM_debug_Print 1 "PerturbExe  = ${PerturbExe}"
  IGCM_debug_Print 1 "PerturbFile = ${PerturbFile}"
  IGCM_debug_Print 1 "PerturbComp = ${PerturbComp}"
  IGCM_debug_Print 1 "PerturbVar  = ${PerturbVar}"
  IGCM_debug_Print 1 "PerturbAmp  = ${PerturbAmp}"

  # A few checks Period case:

  # ... Check PERIODICITE ...
  case ${ensemble_Ens_PERTURB_PERIODICITE} in
  _0_)
    IGCM_debug_Print 1 "periodic start not active"
    CastPeriodicStart=false
    ;;
  *[Yy]|*[Mm])
    CastPeriodicStart=true
    IGCM_debug_Print 1 "Periodic length : ${ensemble_Ens_PERTURB_PERIODICITE}" ;;
  *)
    IGCM_debug_Exit "IGCM_ensemble_CastInit ${ensemble_Ens_PERTURB_PERIODICITE} : invalid PERIODICITE"
    IGCM_debug_Exit "Choose a value in *Y or *M"
    IGCM_debug_Verif_Exit ;;
  esac
# ... Check DUREE ...
  case ${ensemble_Ens_PERTURB_DUREE} in
  *[Yy]|*[Mm])
    IGCM_debug_Print 1 "Periodic duration : ${ensemble_Ens_PERTURB_DUREE}" ;;
  *)
    IGCM_debug_Exit "IGCM_ensemble_CastInit ${ensemble_Ens_PERTURB_DUREE} invalid DUREE"
    IGCM_debug_Exit "Choose a value in choose in *Y or *M"
    IGCM_debug_Verif_Exit ;;
  esac

  # A few checks for the Non-Periodic case:
  DateNum=0
  while [ $DateNum -lt ${#ensemble_Ens_PERTURB_NONPERIODIC[*]} ] ; do

    # - Check DUREE_NONPERIODIC
    case ${ensemble_Ens_PERTURB_DUREE_NONPERIODIC[$DateNum]} in
    _0_)
      IGCM_debug_Print 1 "non-periodic start not active"
      CastNonPeriodicStart=false
      ;;
    *[Yy]|*[Mm])
      IGCM_debug_Print 1 "Non-periodic duration : ${ensemble_Ens_PERTURB_DUREE_NONPERIODIC[$DateNum]}"
      CastNonPeriodicStart=true
      ;;
    *)
      IGCM_debug_Exit "IGCM_ensemble_CastInit ${ensemble_Ens_PERTURB_DUREE_NONPERIODIC[$DateNum]} : invalid DUREE"
      IGCM_debug_Exit "choose in *Y or *M"
      IGCM_debug_Verif_Exit ;;
    esac
    (( DateNum = DateNum + 1 ))
  done

  # Need to know all the restart filename of the component we will apply the noise to
  IGCM_card_DefineArrayFromOption config.card ListOfComponents ${PerturbComp}
  eval compname=\${config_ListOfComponents_${PerturbComp}[0]} > /dev/null 2>&1

  # Target the component's card we apply the noise to
  card=${SUBMIT_DIR}/COMP/${compname}.card

  # Read the restart file list. To be used later
  IGCM_card_DefineArrayFromOption ${card} RestartFiles List
  ListFilesName=${compname}_RestartFiles_List
  eval FileName0=\${${ListFilesName}[0]} > /dev/null 2>&1
  eval NbFiles=\${#${ListFilesName}[@]} > /dev/null 2>&1

  # Check
  IGCM_debug_Print 1 "Nb Restart Files  = ${NbFiles}"

  IGCM_debug_PopStack "IGCM_ensemble_CastInit"
}

function IGCM_ensemble_CastPeriodicStarts
{
  IGCM_debug_PushStack "IGCM_ensemble_CastPeriodicStarts"

  [ ${CastPeriodicStart} = false ] && return

  echo
  IGCM_debug_Print 1 "Manage periodic starts"

#.. Manage periodic starts ..
#   ======================

# ... Loop over DateBegin ...
  eval DateBegin=\${ensemble_Ens_PERTURB_DEBUT_INIT}

  while [ ${DateBegin} -le ${ensemble_Ens_PERTURB_FIN_INIT} ] ; do
    IGCM_date_GetYearMonth ${DateBegin} year month

  # - Determine number of day(s) in PERIODICITE
    PeriodLengthInDays=$( IGCM_date_DaysInCurrentPeriod ${DateBegin} ${ensemble_Ens_PERTURB_PERIODICITE} )

  # - Determine number of day(s) in DUREE
    DureeLengthInDays=$(( $( IGCM_date_DaysInCurrentPeriod ${DateBegin} ${ensemble_Ens_PERTURB_DUREE} ) - 1 ))

  # - Determine DateEnd
    (( DateEnd = $( IGCM_date_AddDaysToGregorianDate ${DateBegin} ${DureeLengthInDays} ) ))

  # - Build directory name
    IGCM_ensemble_CastDirectoryName ${ensemble_Ens_PERTURB_NAME} ${ensemble_Ens_PERTURB_PERIODICITE} $year $month $StartDir

  # - Determine RestartDate
    (( Offset = -1 ))
    (( RestartDate = $( IGCM_date_AddDaysToGregorianDate ${DateBegin} ${Offset} ) ))

    IGCM_debug_Print 2 "${DateBegin} => ${DateEnd} : ${StartDir}"
    echo "${DateBegin} ${DateEnd} ${StartDir}" >> CreatedDir.txt

  # - Create directory for current DateBegin
    if [ ! -d  ${StartDir} ] ; then
      IGCM_sys_Mkdir ${SUBMIT_DIR}/${StartDir}
      IGCM_sys_Cd ${SUBMIT_DIR}/${StartDir}
      ln -s ../../.resol .
      ln -s ../../.libmpi .
      IGCM_sys_Cd ${RUN_DIR}
    fi

  # - Create directory in which to store new restart files
    RestartDir=${STORAGE}/IGCM_IN/${config_UserChoices_TagName}/${StartDir}
    IGCM_sys_MkdirArchive ${RestartDir}

  # - Loop over members
    i=0
    while [ $i -lt ${ensemble_Ens_PERTURB_MEMBER} ] ; do
      MemberDir="${StartDir}${Alpha[$i]}"
      echo
      IGCM_debug_Print 3 "${MemberDir}"

      JobName="Job_${MemberDir}"

    # * Create directory if it doesn't exist and copy/link files
      if [ ! -d  ${SUBMIT_DIR}/${StartDir}/${MemberDir} ] ; then
        IGCM_sys_Mkdir ${SUBMIT_DIR}/${StartDir}/${MemberDir}
        #IGCM_sys_Cp -r COMP/ PARAM/ ${StartDir}/${MemberDir}
        IGCM_sys_Cd ${SUBMIT_DIR}/${StartDir}/${MemberDir}
        ln -s ../../COMP
        ln -s ../../PARAM
        ln -s ../../POST
        IGCM_sys_Cd ${RUN_DIR}
        IGCM_sys_Cp config.card run.card.init ${SUBMIT_DIR}/${StartDir}/${MemberDir}
        IGCM_sys_Cp Job_${ensemble_Ens_PERTURB_NAME} ${SUBMIT_DIR}/${StartDir}/${MemberDir}/${JobName}

        # Dump command to be lauched
        echo "cd ${StartDir}/${MemberDir}/ ;"  >> Qsub.${StartDir}.txt
        echo "${SUBMIT} ${JobName} ; cd -"     >> Qsub.${StartDir}.txt
      fi

    # * Update files : config.card, Job_, COMP/comp.card
      IGCM_ensemble_CastFilesUpdate ${DateBegin} ${DateEnd} ${RestartDate}

    # * Apply noise on restart file
      IGCM_ensemble_CastPerturbFile

      (( i = i + 1 ))
    done

    # Done. Save ${StartDir} submission text file
    IGCM_sys_Cp ${RUN_DIR}/Qsub.${StartDir}.txt ${SUBMIT_DIR}

  # - Next DateBegin
    echo "$DateBegin  $PeriodLengthInDays"
    case ${ensemble_Ens_PERTURB_PERIODICITE} in
    *[Yy]|*[Mm])
      (( DateBegin = $( IGCM_date_AddDaysToGregorianDate ${DateBegin} ${PeriodLengthInDays} ) ))
      ;;
    esac
    echo "New DateBegin = $DateBegin"
    echo "========================================================================"
  done
  IGCM_debug_PopStack "IGCM_ensemble_CastPeriodicStarts"
}

function IGCM_ensemble_CastNonPeriodicStarts
{
  IGCM_debug_PushStack "IGCM_ensemble_CastNonPeriodicStarts"

  #.. Manage non periodic starts => Loop over DateBegin ..
  #   ==========================

  [ ${CastNonPeriodicStart} = false ] && return

  echo
  IGCM_debug_Print 1 "Manage non periodic starts"

  DateNum=0
# ... Loop over ensemble_Ens_PERTURB_NONPERIODIC ...
  echo ">${DateNum}<"
  echo ">${#ensemble_Ens_PERTURB_NONPERIODIC[*]}<"
  while [ $DateNum -lt ${#ensemble_Ens_PERTURB_NONPERIODIC[*]} ] ; do
    DateBegin=${ensemble_Ens_PERTURB_NONPERIODIC[$DateNum]}
    Duree=${ensemble_Ens_PERTURB_DUREE_NONPERIODIC[$DateNum]}
    echo ">${DateBegin}<"
    echo ">${Duree}<"

  # - Determine number of day(s) in DUREE_NONPERIODIC
    IGCM_date_GetYearMonth ${DateBegin} year month
    DureeLengthInDays=$(( $( IGCM_date_DaysInCurrentPeriod ${DateBegin} ${Duree} ) - 1 ))

  # - Build directory name
    echo "========================================================================"
    echo "ensemble_Ens_PERTURB_NAME = ${ensemble_Ens_PERTURB_NAME}"
    IGCM_ensemble_CastDirectoryName ${ensemble_Ens_PERTURB_NAME} ${Duree} $year $month $StartDir

  # - Determine DateEnd
    (( DateEnd = $( IGCM_date_AddDaysToGregorianDate ${DateBegin} ${DureeLengthInDays} ) ))
    echo ">${DateEnd}<"
    echo "tout va bien 1"

  # - Determine RestartDate
    (( Offset = -1 ))
    (( RestartDate = $( IGCM_date_AddDaysToGregorianDate ${DateBegin} ${Offset} ) ))

    IGCM_debug_Print 2 "${DateBegin} => ${DateEnd} : ${StartDir}"

  # -  Does $StartDir already exist ?
    #echo "tout va bien 2" ${StartDir}
    if [ ! -d ${SUBMIT_DIR}/${StartDir} ] ; then
      echo "create dir"
      IGCM_sys_Mkdir ${SUBMIT_DIR}/${StartDir}
      IGCM_sys_Cd ${SUBMIT_DIR}/${StartDir}
      ln -s ../../.resol .
      ln -s ../../.libmpi .
      IGCM_sys_Cd ${RUN_DIR}
      echo "${DateBegin} ${DateEnd} ${StartDir}" >> CreatedDir.txt
    fi
    PeriodDateEnd=$( grep -m1 ${StartDir} CreatedDir.txt | cut -f2 -d\  )

  # - Create directory in which to store new restart files if it does'nt already exist
    RestartDir=${STORAGE}/IGCM_IN/${config_UserChoices_TagName}/${StartDir}
    IGCM_sys_MkdirArchive ${RestartDir}

  # - Loop over members
    i=0
    while [ $i -lt ${ensemble_Ens_PERTURB_MEMBER} ] ; do
      MemberDir="${StartDir}${Alpha[$i]}"
      IGCM_debug_Print 3 "${MemberDir}"

      JobName="Job_${MemberDir}"

    # * Create directory if it doesn't exist and copy files
      if [ ! -d  ${SUBMIT_DIR}/${StartDir}/${MemberDir} ] ; then
        IGCM_sys_Mkdir ${SUBMIT_DIR}/${StartDir}/${MemberDir}
        #IGCM_sys_Cp -r COMP/ PARAM/ ${StartDir}/${MemberDir}
        IGCM_sys_Cd ${SUBMIT_DIR}/${StartDir}/${MemberDir}
        ln -s ../../COMP
        ln -s ../../PARAM
        ln -s ../../POST
        IGCM_sys_Cd ${RUN_DIR}
        IGCM_sys_Cp config.card run.card.init ${SUBMIT_DIR}/${StartDir}/${MemberDir}
        IGCM_sys_Cp Job_${ensemble_Ens_PERTURB_NAME} ${SUBMIT_DIR}/${StartDir}/${MemberDir}/Job_${MemberDir}

        # Dump command to be lauched
        echo "cd ${StartDir}/${MemberDir}/ ;"  >> Qsub.${StartDir}.txt
        echo "${SUBMIT} ${JobName} ; cd -"     >> Qsub.${StartDir}.txt
      fi

    # * Update files : config.card, Job_, COMP/comp.card
      echo "${PeriodDateEnd} ? ${DateEnd}"
      if [ ${PeriodDateEnd} -gt ${DateEnd} ] ; then
        DateEnd=${PeriodDateEnd}
      fi
      IGCM_ensemble_CastFilesUpdate ${DateBegin} ${DateEnd} ${RestartDate}

    # * Apply noise on restart file
      IGCM_ensemble_CastPerturbFile
      (( i = i + 1 ))
    done

    # Done. Save ${StartDir} submission text file
    IGCM_sys_Cp ${RUN_DIR}/Qsub.${StartDir}.txt ${SUBMIT_DIR}

    (( DateNum = DateNum + 1 ))
  done
  IGCM_debug_PopStack "IGCM_ensemble_CastNonPeriodicStarts"
}

function IGCM_ensemble_CastFilesUpdate
{
  IGCM_debug_PushStack "IGCM_ensemble_CastFilesUpdate"

  # Debug Print :
  echo
  IGCM_debug_Print 1 "IGCM_ensemble_CastFilesUpdate :"

  HumanDateBegin=$(   IGCM_date_ConvertFormatToHuman ${1} )
  HumanDateEnd=$(     IGCM_date_ConvertFormatToHuman ${2} )
  HumanRestartDate=$( IGCM_date_ConvertFormatToHuman ${3} )
  # ==> config.card
  IGCM_card_WriteOption ${SUBMIT_DIR}/${StartDir}/${MemberDir}/config.card Ensemble EnsembleRun 'y'
  IGCM_card_WriteOption ${SUBMIT_DIR}/${StartDir}/${MemberDir}/config.card Ensemble EnsembleName ${ensemble_Ens_PERTURB_NAME}
  IGCM_card_WriteOption ${SUBMIT_DIR}/${StartDir}/${MemberDir}/config.card Ensemble EnsembleDate ${StartDir}

  IGCM_card_WriteOption ${SUBMIT_DIR}/${StartDir}/${MemberDir}/config.card UserChoices JobName   ${MemberDir}
  IGCM_card_WriteOption ${SUBMIT_DIR}/${StartDir}/${MemberDir}/config.card UserChoices DateBegin ${HumanDateBegin}
  IGCM_card_WriteOption ${SUBMIT_DIR}/${StartDir}/${MemberDir}/config.card UserChoices DateEnd   ${HumanDateEnd}
  IGCM_card_WriteOption ${SUBMIT_DIR}/${StartDir}/${MemberDir}/config.card Restarts OverRule "n"

  for comp in ${config_ListOfComponents[*]} ; do
    IGCM_card_WriteOption ${SUBMIT_DIR}/${StartDir}/${MemberDir}/config.card ${comp} Restart "y"
    IGCM_card_WriteOption ${SUBMIT_DIR}/${StartDir}/${MemberDir}/config.card ${comp} RestartDate ${HumanRestartDate}
    IGCM_card_WriteOption ${SUBMIT_DIR}/${StartDir}/${MemberDir}/config.card ${comp} RestartJobName ${ensemble_Ens_PERTURB_INITFROM}
    IGCM_card_WriteOption ${SUBMIT_DIR}/${StartDir}/${MemberDir}/config.card ${comp} RestartPath ${ensemble_Ens_PERTURB_INITPATH}
  done

  # ==> Job
  sed -e "s/\(#.*\)${ensemble_Ens_PERTURB_NAME}\( *#.*\)/\1${MemberDir} \2/"            \
      -e "s/\(#.*Script_Output_\)${ensemble_Ens_PERTURB_NAME}\(\.*\)/\1${MemberDir}\2/" \
      -e "s/^PeriodNb=.*/PeriodNb=60/"                                                  \
      ${SUBMIT_DIR}/${StartDir}/${MemberDir}/Job_${MemberDir} > Job_${MemberDir}.tmp
  IGCM_sys_Mv Job_${MemberDir}.tmp ${SUBMIT_DIR}/${StartDir}/${MemberDir}/Job_${MemberDir}

  IGCM_debug_PopStack "IGCM_ensemble_CastFilesUpdate"
}

function IGCM_ensemble_CastDirectoryName
{
  IGCM_debug_PushStack "IGCM_ensemble_CastDirectoryName"

  #.. Debug Print ..
  echo
  IGCM_debug_Print 1 "IGCM_ensemble_CastDirectoryName :"
  echo

  Name=$1
  Duree=$2
  year=$3
  month=$4

  # - Build directory name
  case ${Duree} in
    *Y|*y)
      siecle="$( echo $year | cut -c1-2 )"
      siecle=$( (( $siecle - 18 )) )
      StartYear="${siecle}$( echo $year | cut -c3-4 )"
      StartDir="${Name}${StartYear}"
      ;;
    *M|*m)
      echo $month
      siecle="$( echo $year | cut -c1-2 )"
      siecle=$( (( $siecle - 18 )) )
      StartYear="${siecle}$( echo $year | cut -c3-4 )"
      StartMonth="${AlphaMonth[ (( 10#${month} - 1 )) ]}"
      StartDir="${Name}${StartYear}${StartMonth}"
      ;;
  esac

  IGCM_debug_PopStack "IGCM_ensemble_CastDirectoryName"
}

function IGCM_ensemble_CastPerturbFile
{
  IGCM_debug_PushStack "IGCM_ensemble_CastPerturbFile"

  typeset i i_ j j4 file_out file_out_

  #.. Debug Print ..
  echo
  IGCM_debug_Print 1 "IGCM_ensemble_CastPerturbFile :"

  #.. FileIn ? => RestartDate ..
  DirIn="${ensemble_Ens_PERTURB_INITPATH}/${ensemble_Ens_PERTURB_INITFROM}/${PerturbComp}/Restart"
  DirInTar="${ensemble_Ens_PERTURB_INITPATH}/${ensemble_Ens_PERTURB_INITFROM}/RESTART"
  FileIn="${ensemble_Ens_PERTURB_INITFROM}_${RestartDate}_${PerturbFile}"
  DirOut="${RestartDir}/${MemberDir}/${PerturbComp}/Restart"

  # * Create member restart directory
  IGCM_sys_TestDirArchive ${DirOut}
  RET=$?
  if [ $RET -gt 0 ] ; then
    IGCM_sys_MkdirArchive ${DirOut}
  fi

  FileOut="${MemberDir}_${RestartDate}_${PerturbFile}"
  IGCM_debug_Print 1 "FileIn  = ${DirIn}/${FileIn}"
  IGCM_debug_Print 1 "FileOut = ${DirOut}/${FileOut}.nc"

  IGCM_sys_TestFileArchive ${DirOut}/${FileOut}.nc
  RET=$?
  if [ $RET -gt 0 ] ; then

    # * Look for the restart file we apply the noise to

    # restart file list pertaining to the component we apply the noise to
    # but not being the precise restart file we will apply the noise to
    unset OtherFileInList
    # generic restart filename list (like flxat, sstoc, restart, ...)
    unset OtherGenericList

    if ( [ X${FileName0} != X${NULL_STR} ] && [ X${FileName0} != XNONE ] ) ; then
      (( i=0 ))
      until [ $i -ge ${NbFiles} ]; do

        (( i_ = i+1 ))
        eval file_out_=\${${ListFilesName}[$i_]} > /dev/null 2>&1
        eval file_out=${file_out_}

        generic_restart_file_name_out=$( basename ${file_out} .nc )

        if [ ! ${generic_restart_file_name_out} = ${PerturbFile} ] ; then
          set +A OtherFileInList ${OtherFileInList[*]} ${PerturbComp}_${ensemble_Ens_PERTURB_INITFROM}_${RestartDate}_${generic_restart_file_name_out}\*.nc
          set +A OtherGenericList ${OtherGenericList[*]} ${generic_restart_file_name_out}
        fi

        (( i=i+3 ))
      done
    fi
    # How many restart files other than the one we will apply the noise to
    NbOtherFiles=${#OtherGenericList[*]}

    ##########################
    # TO BE A FUNCTION BEGIN #
    ##########################

    if [ $( IGCM_sys_TestFileBuffer ${DirIn}/${FileIn}*.nc ; echo $? ) = 0 ] ; then
      IGCM_debug_Print 3 "Buffered restart"
      Buffered=true
      Archived=false
      Tared=false
      nb_restart_file=$(IGCM_sys_CountFileBuffer ${DirIn}/${FileIn}_????.nc)
    elif [ $( IGCM_sys_TestFileArchive ${DirIn}/${FileIn}*.nc ; echo $? ) = 0 ] ; then
      IGCM_debug_Print 3 "Archived restart"
      Buffered=false
      Archived=true
      Tared=false
      nb_restart_file=$(IGCM_sys_CountFileArchive ${DirIn}/${FileIn}_????.nc)
    else
      IGCM_debug_Print 3 "Tared restart"
      Buffered=false
      Archived=false
      Tared=true

      # Look for the tar file we want if we did not found it already
      for PotentialTarFile in $( find ${ensemble_Ens_PERTURB_INITPATH}/${ensemble_Ens_PERTURB_INITFROM}/RESTART -name "${ensemble_Ens_PERTURB_INITFROM}_*_restart.tar" -print ) ; do
        IsMatching=$( echo ${PotentialTarFile##*/} | sed "s:^${ensemble_Ens_PERTURB_INITFROM}_::" | sed "s:\.restart\.tar$::" | gawk -F_ -v restartdate=${RestartDate} '{if (($1 < restartdate) && ($2 >= restartdate)) {print $1"_"$2}}' )
        if [ ! X${IsMatching} = X ] ; then
          TarFileFound=${PotentialTarFile}
          break
        fi
      done

      # Extract relevant restart files
      IGCM_debug_Print 1 "tar xvf ${TarFileFound} ${PerturbComp}_${FileIn}*.nc ${OtherFileInList[*]}"
      tar xvf ${TarFileFound} ${PerturbComp}_${FileIn}*.nc ${OtherFileInList[*]}
      nb_restart_file=$( IGCM_sys_CountFileBuffer ${PerturbComp}_${FileIn}_????.nc )
    fi

    # Move around and perturb restart files so as to be able to start hindcast/forecast simulation members
    if [ ${nb_restart_file} -gt 1 ] ; then
      j=0
      until [ $j -ge ${nb_restart_file} ]; do
        j4=${j}
        if [ X${Buffered} = Xtrue ] ; then
          IGCM_sys_GetBuffer ${DirIn}/${FileIn}_${j4}.nc ${RUN_DIR}/${FileOut}_${j4}.nc

          cd ${DirOut}
          for generic in ${OtherGenericList[*]} ; do
            ln -s ${DirIn}/${ensemble_Ens_PERTURB_INITFROM}_${RestartDate}_${generic}.nc ${MemberDir}_${RestartDate}_${generic}_${j4}.nc
          done
          cd -

        elif [ X${Archived} = Xtrue ] ; then
          IGCM_sys_Get ${DirIn}/${FileIn}_${j4}.nc ${RUN_DIR}/${FileOut}_${j4}.nc

          for generic in ${OtherGenericList[*]} ; do
            IGCM_sys_RshArchive "cd ${DirOut} ; ln -s ${DirIn}/${ensemble_Ens_PERTURB_INITFROM}_${RestartDate}_${generic}.nc ${MemberDir}_${RestartDate}_${generic}_${j4}.nc"
          done

        elif [ X${Tared} = Xtrue ] ; then
          IGCM_sys_Mv ${PerturbComp}_${FileIn}_${j4}.nc ${RUN_DIR}/${FileOut}_${j4}.nc

          for generic in ${OtherGenericList[*]} ; do
            IGCM_sys_Mv ${PerturbComp}_${ensemble_Ens_PERTURB_INITFROM}_${RestartDate}_${generic}.nc ${DirOut}/${MemberDir}_${RestartDate}_${generic}_${j4}.nc
          done

        fi
        (( j=j+1 ))
      done
    else
      if [ X${Buffered} = Xtrue ] ; then
        IGCM_sys_GetBuffer ${DirIn}/${FileIn}.nc ${RUN_DIR}/${FileOut}.nc

        cd ${DirOut}
        for generic in ${OtherGenericList[*]} ; do
          ln -s ${DirIn}/${ensemble_Ens_PERTURB_INITFROM}_${RestartDate}_${generic}.nc ${MemberDir}_${RestartDate}_${generic}.nc
        done
        cd -

      elif [ X${Archived} = Xtrue ] ; then
        IGCM_sys_Get ${DirIn}/${FileIn}.nc ${RUN_DIR}/${FileOut}.nc

        for generic in ${OtherGenericList[*]} ; do
          IGCM_sys_RshArchive "cd ${DirOut} ; ln -s ${DirIn}/${ensemble_Ens_PERTURB_INITFROM}_${RestartDate}_${generic}.nc ${MemberDir}_${RestartDate}_${generic}.nc"
        done

      elif [ X${Tared} = Xtrue ] ; then
        IGCM_sys_Mv ${PerturbComp}_${FileIn}.nc ${RUN_DIR}/${FileOut}.nc

        for generic in ${OtherGenericList[*]} ; do
          IGCM_sys_Mv ${PerturbComp}_${ensemble_Ens_PERTURB_INITFROM}_${RestartDate}_${generic}.nc ${DirOut}/${MemberDir}_${RestartDate}_${generic}.nc
        done

      fi
    fi

    ########################
    # TO BE A FUNCTION END #
    ########################

    IGCM_sys_Chmod 644 ${RUN_DIR}/${FileOut}.nc
    IGCM_debug_Print 1 "${PerturbExe} ${RUN_DIR}/${FileOut}.nc ${PerturbVar} ${PerturbAmp}"
    echo
    ${PerturbExe} ${RUN_DIR}/${FileOut}.nc ${PerturbVar} ${PerturbAmp} > /dev/null 2>&1

    if [ $? -ne 0 ] ; then
      echo "Abend $( basename ${PerturbExe} )"
      return
    fi
    IGCM_sys_Put_Out ${RUN_DIR}/${FileOut}.nc ${DirOut}/ 644
  fi

  #.. Update config.card..
  IGCM_card_WriteOption ${SUBMIT_DIR}/${StartDir}/${MemberDir}/config.card ${PerturbComp} Restart "y"
  IGCM_card_WriteOption ${SUBMIT_DIR}/${StartDir}/${MemberDir}/config.card ${PerturbComp} RestartDate    ${HumanRestartDate}
  IGCM_card_WriteOption ${SUBMIT_DIR}/${StartDir}/${MemberDir}/config.card ${PerturbComp} RestartJobName ${MemberDir}
  IGCM_card_WriteOption ${SUBMIT_DIR}/${StartDir}/${MemberDir}/config.card ${PerturbComp} RestartPath    ${RestartDir}/

  IGCM_debug_PopStack "IGCM_ensemble_CastPerturbFile"
}
